import logging as log
log.basicConfig(level=log.INFO)

import os
import collections
import numpy as np
from .xray import azav
from .xray import dataReduction
from .xray import utils

storage_extension = ".npz"

def _conv(x):
    try:
        x = float(x)
    except:
        x = np.nan
    return x

def readDelayFromDiagnostic(fname):
  """ return an ordered dict dictionary of filename; for each key a rounded
      value of delay is associated """
  if os.path.isdir(fname): fname += "/diagnostics.log"
  data = np.genfromtxt(fname,usecols=(2,3),\
          dtype=None,converters={3: lambda x: _conv(x)},
          names = ['fname','delay'])
  files = data['fname'].astype(str)
  delays = data['delay']
  # skip lines that cannot be interpreted as float (like done, etc)
  idx_ok = np.isfinite( delays )
  files = files[idx_ok]
  delays = delays[idx_ok]
  delays = np.round(delays.astype(float),12)
  return collections.OrderedDict( zip(files,delays) )


def doFolder_azav(folder,nQ=1500,force=False,mask=None,saveChi=True,
  poni='auto',storageFile='auto'):
  """ very small wrapper around azav.doFolder, essentially just reading
      the diagnostics.log """

  diag = dict( delays = readDelayFromDiagnostic(folder) )
  if storageFile == 'auto' : storageFile = folder +  "/" + "pyfai_1d" + storage_extension
  return azav.doFolder(folder,files="*.edf*",nQ=nQ,force=force,mask=mask,
    saveChi=saveChi,poni=poni,storageFile=storageFile,diagnostic=diag)

def doFolder_dataRed(folder,monitor=None,funcForEveraging=np.nanmean,errFilter=True):
  data = utils.data_storage(folder)
  scan = data.delays
  q    = data.q
  i    = data.data
  diffs = dataReduction.calcTimeResolvedSignal(scan,i,q=q,reference="min",
          monitor=monitor,funcForEveraging=funcForEveraging)
  if funcForEveraging == np.nanmean and errFilter: 
    diffs = dataReduction.errorFilter(diffs)
  dataReduction.saveTxt(folder,diffs,info=data.pyfai_info)
  diffs.save(folder + "/" + "diffs" + storage_extension)
  return diffs
